/*

    wad.h: definition of various structures for the DOOM wadfile.
    Copyright (C) 1994  Robert H. Forsman Jr.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <stdio.h>

/* these may have to be changed if you compile on a non-SUN */

typedef long INT32;
typedef short INT16;
typedef char INT8;

#if 1
#define strncasecmp	strnicmp
#define strcasecmp	stricmp
#endif

/* macros to fix up byte order.  Sun is network byte order, not Intel
   byte order.  On an Intel CPU, these macros would be noops */

#if 0		/* put some intel-specific preprocessor symbol there */
#define Itohl(i) (INT32)((((i)<<24)&0xff000000L) | \
				 (((i)<< 8)&  0xff0000L) | \
				 (((i)>> 8)&    0xff00) | \
				 (((i)>>24)&      0xff))
#define Itohs(i) (INT32)((((i)<< 8)&0xff00) | \
			 (((i)>> 8)&  0xff))
#define htoIl(i) (INT32)((((i)<<24)&0xff000000L)| \
				 (((i)<< 8)&  0xff0000L) | \
				 (((i)>> 8)&    0xff00) | \
				 (((i)>>24)&      0xff))
#define htoIs(i) (INT32)((((i)<< 8)&0xff00) | \
			 (((i)>> 8)&  0xff))
#else
#define	Itohl(i) ((INT32)((i)&0xffffffffL))
#define	Itohs(i) ((INT16)((i)&0xffff))
#define	htoIl(i) ((INT32)((i)&0xffffffffL))
#define	htoIs(i) ((INT16)((i)&0xffff))
#endif

struct wadfile {
    char	magic[4];
    INT32	nentries;
    INT32 offset;
};

struct dw_direntry {
    INT32	offset;
    INT32	length;
    char	name[8];	/* null padded, but not necessarily terminated */
};

struct dw_thing {
    INT16	x;
    INT16	y;
    INT16	dir;
    INT16	type;
    INT16	attr;
};

struct dw_vertex {
    INT16	x, y;
};

struct dw_linedef {
    INT16	v[2];
    INT16	attr;
    INT16	type;
    INT16	tag;
    INT16	sidedefs[2];
};

struct dw_sidedef {
    INT16	x, y;
    char	upper[8];
    char	lower[8];
    char	normal[8];
    INT16	sector;
};


struct dw_sector {
    INT16	floor, ceiling;
    char	ftexture[8], ctexture[8];
    INT16	light;
    INT16	special;
    INT16	tag;
};


struct dw_texture_list {
    INT32	ntextures;
    INT32	offsets[1/*ntextures*/];
};

struct dw_one_texture {
    char	name[8];
    INT16	zero1, zero2;	/* always 0 */
    INT16	width,height;
    INT16	zero3, zero4;	/* always 0 */
    INT16	npatches;
    struct dw_patchdesc {
	INT16	xoff, yoff;
	INT16	pnames_idx;
	INT16	stepdir;	/* always 1 according to dmspec13 */
	INT16	colormap;	/* always 0 */
    } patches[1];
};

struct dw_pnames {
    INT32	count;
    char	names[1][8];
};


void upcopy8(char *, char *);
void write8(char *, FILE *);
FILE *fopen_b(char *, int forwrite, int binary);
void fclose_b(FILE *);
void write_I16(int, FILE*);
void write_I32(int, FILE*);

struct wadhandle {
    FILE	*fp;
    long	basepos;
	 INT32 nentries;
    struct dw_direntry	*entries;
	 INT32	space;			/* how many entries we have space for */

    /* for writing mission data */
    int inmission;
    long bitvec;
#define MIS_THINGS	0
#define MIS_LINEDEFS	1
#define MIS_SIDEDEFS	2
#define MIS_VERTEXES	3
#define MIS_SEGS	4
#define MIS_SSECTORS	5
#define MIS_NODES	6
#define MIS_SECTORS	7
#define MIS_REJECT	8
#define MIS_BLOCKMAP	9
#define MIS_COUNT	10
};

extern char *mapres[MIS_COUNT];

struct wadhandle *open_wadfile(FILE *);
void scan_wadfile(struct wadhandle*);
void *get_resource_from_wadfile(struct wadhandle*, INT32*, char *);
int write_entry_to_wadfile(struct wadhandle*, void*, INT32, char*);
int close_wadfile(struct wadhandle*);
